/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.baseConfig.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.org.entity.BizOrg;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.baseConfig.param.*;
import vip.xiaonuo.modular.base.baseConfig.service.BaseConfigService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 基础设置控制器
 *
 * @author newspaper
 * @date  2023/12/08 10:22
 */
@Api(tags = "基础设置控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class BaseConfigController {

    @Resource
    private BaseConfigService baseConfigService;

    /**
     * 获取基础设置分页
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取基础设置树")
    @GetMapping("/base/baseConfig/tree")
    public CommonResult<List<Tree<String>>> tree(OrgIdParam orgIdParam) {
        return CommonResult.data(baseConfigService.tree(orgIdParam));
    }

    /**
     * 添加基础设置
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加基础设置")
    @CommonLog("添加基础设置")
    @PostMapping("/base/baseConfig/add")
    public CommonResult<String> add(@RequestBody @Valid BaseConfigAddParam baseConfigAddParam) {
        baseConfigService.add(baseConfigAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑基础设置
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑基础设置")
    @CommonLog("编辑基础设置")
    @PostMapping("/base/baseConfig/edit")
    public CommonResult<String> edit(@RequestBody @Valid BaseConfigEditParam baseConfigEditParam) {
        baseConfigService.edit(baseConfigEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除基础设置
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除基础设置")
    @CommonLog("删除基础设置")
    @PostMapping("/base/baseConfig/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BaseConfigIdParam> baseConfigIdParamList) {
        baseConfigService.delete(baseConfigIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取基础设置详情
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取基础设置详情")
    @GetMapping("/base/baseConfig/detail")
    public CommonResult<BaseConfig> detail(@Valid BaseConfigIdParam baseConfigIdParam) {
        return CommonResult.data(baseConfigService.detail(baseConfigIdParam));
    }

    /**
     * 根据父ID获取下级配置
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("根据父ID获取下级配置")
    @GetMapping("/base/baseConfig/listChild")
    public CommonResult<Page<BaseConfig>> listChild(@Valid BaseConfigListChildParam baseConfigListChildParam) {
        return CommonResult.data(baseConfigService.listChild(baseConfigListChildParam));
    }

    /**
     * 获取当前用户组织id
     *
     * @author newspaper
     * @date  2023/12/08 10:22
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取组织id")
    @GetMapping("/base/baseConfig/listOrgId")
    public CommonResult<List<BizOrg>> listOrgId() {
        return CommonResult.data(baseConfigService.listOrgId());
    }


}
