/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.baseSeason.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.base.baseSeason.entity.BaseSeason;
import vip.xiaonuo.modular.base.baseSeason.param.*;
import vip.xiaonuo.modular.base.baseSeason.service.BaseSeasonService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.text.ParseException;

/**
 * 季节控制器
 *
 * @author newspaper
 * @date  2023/12/18 11:14
 */
@Api(tags = "季节控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class BaseSeasonController {

    @Resource
    private BaseSeasonService baseSeasonService;

    /**
     * 获取季节详情
     *
     * @author newspaper
     * @date  2023/12/18 11:14
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取季节详情")
    @CommonLog("获取季节详情")
    @GetMapping("/base/baseSeason/getSeason")
    public CommonResult<BaseSeason> getSeason(BaseSeasonTypeParam baseSeasonTypeParam) throws ParseException {
        return CommonResult.data(baseSeasonService.getSeason(baseSeasonTypeParam));
    }


    /**
     * 编辑季节
     *
     * @author newspaper
     * @date  2023/12/18 11:14
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("编辑季节")
    @CommonLog("编辑季节")
    @PostMapping("/base/baseSeason/edit")
    public CommonResult<String> edit(@RequestBody @Valid BaseSeasonEditParam baseSeasonEditParam) throws ParseException {
        baseSeasonService.edit(baseSeasonEditParam);
        return CommonResult.ok();
    }

    /**
     * 获取启用季节
     *
     * @author newspaper
     * @date  2023/12/18 11:14
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("获取启用季节")
    @CommonLog("获取启用季节")
    @GetMapping("/base/baseSeason/getEnableSeason")
    public CommonResult<Integer> getEnableSeason(OrgIdParam orgIdParam) {
        return CommonResult.data(baseSeasonService.getEnableSeason(orgIdParam));
    }
}
