/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.department.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fhs.core.trans.anno.Trans;
import com.fhs.core.trans.constant.TransType;
import com.fhs.core.trans.vo.TransPojo;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.common.pojo.CommonEntity;
import vip.xiaonuo.dev.modular.file.entity.DevFile;
import vip.xiaonuo.sys.modular.user.entity.SysUser;

import java.util.Date;

/**
 * 部门实体
 *
 * @author newspaper
 * @date  2023/12/13 08:44
 **/
@Getter
@Setter
@TableName("department")
public class Department extends CommonEntity implements TransPojo {

    /** ID */
    @TableId
    @ApiModelProperty(value = "ID", position = 1)
    private String id;

    /** 部门名称 */
    @ApiModelProperty(value = "部门名称", position = 2)
    private String departmentName;

    /** 部门照片 */
    @ApiModelProperty(value = "部门照片", position = 3)
    private String departmentImgUrl;

    /** 上级部门ID */
    @ApiModelProperty(value = "上级部门ID", position = 4)
    @Trans(type = TransType.SIMPLE, target = Department.class, fields = "departmentName", alias = "parent", ref = "parentName")
    private String parentId;

    /** 所在区域 */
    @ApiModelProperty(value = "所在区域", position = 5)
    private String departmentRegion;

    /** 详细地址 */
    @ApiModelProperty(value = "详细地址", position = 6)
    private String detailLocation;

    /** 部门电话 */
    @ApiModelProperty(value = "部门电话", position = 7)
    private String departmentPhone;

    /** 部门负责人ID */
    @ApiModelProperty(value = "部门负责人ID", position = 8)
    @Trans(type = TransType.SIMPLE, target = SysUser.class, fields = "name", alias = "manager", ref = "managerName")
    private String departmentManager;

    /** 描述 */
    @ApiModelProperty(value = "描述", position = 9)
    private String remarks;

    /** 排序码 */
    @ApiModelProperty(value = "排序码", position = 10)
    private Integer sortCode;

    /** 组织ID */
    @ApiModelProperty(value = "组织ID", position = 11)
    private String orgId;

    /** 负责人名称 */
    @ApiModelProperty(value = "负责人", position = 12)
    @TableField(exist = false)
    private String managerName;

    /** 上级部门 */
    @ApiModelProperty(value = "上级部门", position = 13)
    @TableField(exist = false)
    private String parentName;
}
