/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.department.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.biz.modular.user.entity.BizUser;
import vip.xiaonuo.modular.base.department.entity.Department;
import vip.xiaonuo.modular.base.department.param.*;
import vip.xiaonuo.sys.modular.user.entity.SysUser;

import java.io.IOException;
import java.util.List;

/**
 * 部门Service接口
 *
 * @author newspaper
 * @date  2023/12/13 08:44
 **/
public interface DepartmentService extends IService<Department> {

    /**
     * 获取部门分页
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    Page<Department> page(DepartmentPageParam departmentPageParam);

    /**
     * 添加部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    void add(String departmentName, String parentId, String departmentRegion, String detailLocation, String departmentPhone, String departmentManager, String remarks, Integer sortCode, MultipartFile departmentImgUrl,String orgId) throws IOException;

    /**
     * 编辑部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    void edit(String id,String departmentName, String parentId, String departmentRegion, String detailLocation, String departmentPhone, String departmentManager, String remarks, Integer sortCode, MultipartFile departmentImgUrl) throws IOException;

    /**
     * 删除部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    void delete(List<DepartmentIdParam> departmentIdParamList);

    /**
     * 获取部门详情
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    Department detail(DepartmentIdParam departmentIdParam);

    /**
     * 获取部门详情
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     **/
    Department queryEntity(String id);

    List<Tree<String>> tree(DepartmentOrgIdParam departmentOrgIdParam);

    Page<Department> listChild(DepartmentChildParam departmentChildParam);

    List<BizUser> userSelector(DepartmentOrgIdParam departmentOrgIdParam);
}
