/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.inventoryUse.param;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 物资领用添加参数
 *
 * @author newspaper
 * @date  2023/12/11 10:43
 **/
@Getter
@Setter
public class InventoryUseAddParam extends InventoryUseOrgIdParam{

    /** 物资编号 */
    @ApiModelProperty(value = "物资编号", required = true, position = 2)
    @NotBlank(message = "itemCode不能为空")
    private String itemCode;

    /** 物资名称 */
    @ApiModelProperty(value = "物资名称", required = true, position = 3)
    @NotBlank(message = "itemName不能为空")
    private String itemName;

    /** 物资类别 */
    @ApiModelProperty(value = "物资类别", required = true, position = 4)
    @NotNull(message = "itemType不能为空")
    private String itemType;

    /** 品牌 */
    @ApiModelProperty(value = "品牌", required = true, position = 5)
    @NotBlank(message = "brand不能为空")
    private String brand;

    /** 型号规格 */
    @ApiModelProperty(value = "型号规格", required = true, position = 6)
    @NotBlank(message = "specification不能为空")
    private String specification;

    /** 单位 */
    @ApiModelProperty(value = "单位", required = true, position = 7)
    @NotBlank(message = "unit不能为空")
    private String unit;

    /** 仓库ID */
    @ApiModelProperty(value = "仓库ID", required = true, position = 8)
    @NotBlank(message = "仓库ID不能为空")
    private String warehouseId;

    /** 领用时间 */
    @ApiModelProperty(value = "领用时间", required = true, position = 9)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @NotNull(message = "useTime不能为空")
    private Date useTime;

    /** 领用数量 */
    @ApiModelProperty(value = "领用数量", required = true, position = 10)
    @NotNull(message = "useQuantity不能为空")
    private BigDecimal useQuantity;

    /** 领用部门ID */
    @ApiModelProperty(value = "领用部门ID", required = true, position = 11)
    @NotBlank(message = "useDepartmentId不能为空")
    private String useDepartmentId;

    /** 领用人ID */
    @ApiModelProperty(value = "领用人ID", required = true, position = 12)
    @NotBlank(message = "userId不能为空")
    private String userId;

    /** 备注 */
    @ApiModelProperty(value = "备注", position = 13)
    private String remarks;

}
