/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.inventoryUse.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.listener.CommonDataChangeEventCenter;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.core.enums.BaseDataTypeEnum;
import vip.xiaonuo.dev.modular.file.enums.DevFileEngineTypeEnum;
import vip.xiaonuo.dev.modular.file.service.DevFileService;
import vip.xiaonuo.modular.base.inventoryUse.entity.InventoryUse;
import vip.xiaonuo.modular.base.inventoryUse.entity.vo.InventoryUseDetailVo;
import vip.xiaonuo.modular.base.inventoryUse.entity.vo.InventoryUseVo;
import vip.xiaonuo.modular.base.inventoryUse.mapper.InventoryUseMapper;
import vip.xiaonuo.modular.base.inventoryUse.param.*;
import vip.xiaonuo.modular.base.inventoryUse.service.InventoryUseService;
import vip.xiaonuo.modular.bi.utils.UploadImage;

import javax.validation.constraints.NotBlank;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 物资领用Service接口实现类
 *
 * @author newspaper
 * @date  2023/12/11 10:43
 **/
@Service
public class InventoryUseServiceImpl extends ServiceImpl<InventoryUseMapper, InventoryUse> implements InventoryUseService {

    @Autowired
    private UploadImage uploadImage;

    @Autowired
    private InventoryUseMapper inventoryUseMapper;

    @Override
    public Page<InventoryUse> page(InventoryUsePageParam inventoryUsePageParam) {
        QueryWrapper<InventoryUse> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(inventoryUsePageParam.getItemName())) {
            queryWrapper.like("item_name", inventoryUsePageParam.getItemName()).orderByDesc("use_time");;
        }
        queryWrapper.lambda().eq(InventoryUse::getOrgId,inventoryUsePageParam.getOrgId());
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(InventoryUseAddParam inventoryUseAddParam) {
        InventoryUse inventoryUse = BeanUtil.toBean(inventoryUseAddParam, InventoryUse.class);
        this.save(inventoryUse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(InventoryUseEditParam inventoryUseEditParam) {
        InventoryUse inventoryUse = this.queryEntity(inventoryUseEditParam.getId());
        BeanUtil.copyProperties(inventoryUseEditParam, inventoryUse);
        this.updateById(inventoryUse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<InventoryUseIdParam> inventoryUseIdParamList) {
        List<String> toDeleteInventoryUseIdList = CollStreamUtil.toList(inventoryUseIdParamList, InventoryUseIdParam::getId);
        // 执行删除
        this.removeByIds(toDeleteInventoryUseIdList);
    }

    @Override
    public InventoryUse detail(InventoryUseIdParam inventoryUseIdParam) {
        return queryEntity(inventoryUseIdParam.getId());
    }

    @Override
    public InventoryUse queryEntity(String id) {
        InventoryUse inventoryUse = this.getById(id);
        if(ObjectUtil.isEmpty(inventoryUse)) {
            throw new CommonException("物资领用不存在，id值为：{}", id);
        }
        return inventoryUse;
    }

    @Override
    public void revert(MultipartFile returnImgUrl, String id, String returnTime, String remarks) throws ParseException, IOException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        InventoryUse inventoryUse = this.getById(id);
        String url = uploadImage.getImageCom(returnImgUrl);
        if (url == "上传失败"){
            throw new CommonException("图片上传失败！");
        }
        inventoryUse.setReturnTime(dateFormat.parse(returnTime));
        inventoryUse.setReturnImgUrl(url);
        inventoryUse.setRemarks(remarks);
        this.updateById(inventoryUse);
    }

}
