/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigbatch.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.bi.pigbatch.entity.PigBatch;
import vip.xiaonuo.modular.bi.pigbatch.mapper.PigBatchMapper;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchAddParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchEditParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchIdParam;
import vip.xiaonuo.modular.bi.pigbatch.param.PigBatchPageParam;
import vip.xiaonuo.modular.bi.pigbatch.service.PigBatchService;

import java.util.List;

/**
 * 猪只批次Service接口实现类
 *
 * @author wang
 * @date  2023/12/06 14:53
 **/
@Service
public class PigBatchServiceImpl extends ServiceImpl<PigBatchMapper, PigBatch> implements PigBatchService {
    @Autowired
    private PigBatchMapper batchMapper;

    @Override
    public Page<PigBatch> page(PigBatchPageParam pigBatchPageParam) {
        QueryWrapper<PigBatch> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigBatchPageParam.getBatchCode())) {
            queryWrapper.lambda().like(PigBatch::getBatchCode, pigBatchPageParam.getBatchCode());
        }
        if(ObjectUtil.isAllNotEmpty(pigBatchPageParam.getSortField(), pigBatchPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigBatchPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigBatchPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigBatchPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigBatch::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(PigBatchAddParam pigBatchAddParam) {
        QueryWrapper<PigBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", pigBatchAddParam.getOrgId()).eq("batch_code", pigBatchAddParam.getBatchCode());
        if (ObjectUtil.isEmpty(batchMapper.selectOne(queryWrapper))) {
            PigBatch pigBatch = BeanUtil.toBean(pigBatchAddParam, PigBatch.class);
            this.save(pigBatch);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "批次号已存在，请重新添加！", false);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(PigBatchEditParam pigBatchEditParam) {
        PigBatch pigBatch = this.queryEntity(pigBatchEditParam.getId());
        if (ObjectUtil.isNotEmpty(pigBatchEditParam.getBatchCode())) {
            QueryWrapper<PigBatch> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id", pigBatch.getOrgId())
                    .eq(StringUtils.isNotBlank(pigBatchEditParam.getBatchCode()),"batch_code",pigBatchEditParam.getBatchCode())
                    .ne("id", pigBatchEditParam.getId());
            if (ObjectUtil.isEmpty(batchMapper.selectOne(queryWrapper))) {
                BeanUtil.copyProperties(pigBatchEditParam, pigBatch);
                this.updateById(pigBatch);
                return CommonResult.ok();
            } else {
                return new CommonResult(500, "耳标号已存在！", false);
            }
        } else {
            BeanUtil.copyProperties(pigBatchEditParam, pigBatch);
            this.updateById(pigBatch);
            return CommonResult.ok();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigBatchIdParam> pigBatchIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigBatchIdParamList, PigBatchIdParam::getId));
    }

    @Override
    public PigBatch detail(PigBatchIdParam pigBatchIdParam) {
        return this.queryEntity(pigBatchIdParam.getId());
    }

    @Override
    public PigBatch queryEntity(Integer id) {
        PigBatch pigBatch = this.getById(id);
        if(ObjectUtil.isEmpty(pigBatch)) {
            throw new CommonException("猪只批次不存在，id值为：{}", id);
        }
        return pigBatch;
    }
}
