/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.clean.billpersonneladmission.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.clean.billpersonneladmission.entity.BillPersonnelAdmission;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionAddParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionEditParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionIdParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionPageParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.service.BillPersonnelAdmissionService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 入场申请控制器
 *
 * @author wang
 * @date  2024/01/03 09:27
 */
@Api(tags = "入场申请控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class BillPersonnelAdmissionController {

    @Resource
    private BillPersonnelAdmissionService billPersonnelAdmissionService;

    /**
     * 获取入场申请分页
     *
     * @author wang
     * @date  2024/01/03 09:27
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取入场申请分页")
    @SaCheckPermission("/clean/billpersonneladmission/page")
    @GetMapping("/clean/billpersonneladmission/page")
    public CommonResult<Page<BillPersonnelAdmission>> page(BillPersonnelAdmissionPageParam billPersonnelAdmissionPageParam) {
        return CommonResult.data(billPersonnelAdmissionService.page(billPersonnelAdmissionPageParam));
    }

    /**
     * 添加入场申请
     *
     * @author wang
     * @date  2024/01/03 09:27
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加入场申请")
    @CommonLog("添加入场申请")
    @SaCheckPermission("/clean/billpersonneladmission/add")
    @PostMapping("/clean/billpersonneladmission/add")
    public CommonResult<String> add(@RequestBody @Valid BillPersonnelAdmissionAddParam billPersonnelAdmissionAddParam) {
        billPersonnelAdmissionService.add(billPersonnelAdmissionAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑入场申请
     *
     * @author wang
     * @date  2024/01/03 09:27
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑入场申请")
    @CommonLog("编辑入场申请")
    @SaCheckPermission("/clean/billpersonneladmission/edit")
    @PostMapping("/clean/billpersonneladmission/edit")
    public CommonResult<String> edit(@RequestBody @Valid BillPersonnelAdmissionEditParam billPersonnelAdmissionEditParam) {
        billPersonnelAdmissionService.edit(billPersonnelAdmissionEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除入场申请
     *
     * @author wang
     * @date  2024/01/03 09:27
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除入场申请")
    @CommonLog("删除入场申请")
    @SaCheckPermission("/clean/billpersonneladmission/delete")
    @PostMapping("/clean/billpersonneladmission/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BillPersonnelAdmissionIdParam> billPersonnelAdmissionIdParamList) {
        billPersonnelAdmissionService.delete(billPersonnelAdmissionIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取入场申请详情
     *
     * @author wang
     * @date  2024/01/03 09:27
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取入场申请详情")
    @SaCheckPermission("/clean/billpersonneladmission/detail")
    @GetMapping("/clean/billpersonneladmission/detail")
    public CommonResult<BillPersonnelAdmission> detail(@Valid BillPersonnelAdmissionIdParam billPersonnelAdmissionIdParam) {
        return CommonResult.data(billPersonnelAdmissionService.detail(billPersonnelAdmissionIdParam));
    }
}
