/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.clean.billpersonneladmission.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.clean.billpersonneladmission.entity.BillPersonnelAdmission;
import vip.xiaonuo.modular.clean.billpersonneladmission.mapper.BillPersonnelAdmissionMapper;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionAddParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionEditParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionIdParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.param.BillPersonnelAdmissionPageParam;
import vip.xiaonuo.modular.clean.billpersonneladmission.service.BillPersonnelAdmissionService;

import java.util.List;

/**
 * 入场申请Service接口实现类
 *
 * @author wang
 * @date  2024/01/03 09:27
 **/
@Service
public class BillPersonnelAdmissionServiceImpl extends ServiceImpl<BillPersonnelAdmissionMapper, BillPersonnelAdmission> implements BillPersonnelAdmissionService {

    @Override
    public Page<BillPersonnelAdmission> page(BillPersonnelAdmissionPageParam billPersonnelAdmissionPageParam) {
        QueryWrapper<BillPersonnelAdmission> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(billPersonnelAdmissionPageParam.getStartVistitDate()) && ObjectUtil.isNotEmpty(billPersonnelAdmissionPageParam.getEndVistitDate())) {
            queryWrapper.lambda().between(BillPersonnelAdmission::getVistitDate, billPersonnelAdmissionPageParam.getStartVistitDate(), billPersonnelAdmissionPageParam.getEndVistitDate());
        }
        if(ObjectUtil.isAllNotEmpty(billPersonnelAdmissionPageParam.getSortField(), billPersonnelAdmissionPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(billPersonnelAdmissionPageParam.getSortOrder());
            queryWrapper.orderBy(true, billPersonnelAdmissionPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(billPersonnelAdmissionPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(BillPersonnelAdmission::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BillPersonnelAdmissionAddParam billPersonnelAdmissionAddParam) {
        BillPersonnelAdmission billPersonnelAdmission = BeanUtil.toBean(billPersonnelAdmissionAddParam, BillPersonnelAdmission.class);
        this.save(billPersonnelAdmission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BillPersonnelAdmissionEditParam billPersonnelAdmissionEditParam) {
        BillPersonnelAdmission billPersonnelAdmission = this.queryEntity(billPersonnelAdmissionEditParam.getId().toString());
        BeanUtil.copyProperties(billPersonnelAdmissionEditParam, billPersonnelAdmission);
        this.updateById(billPersonnelAdmission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BillPersonnelAdmissionIdParam> billPersonnelAdmissionIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(billPersonnelAdmissionIdParamList, BillPersonnelAdmissionIdParam::getId));
    }

    @Override
    public BillPersonnelAdmission detail(BillPersonnelAdmissionIdParam billPersonnelAdmissionIdParam) {
        return this.queryEntity(billPersonnelAdmissionIdParam.getId().toString());
    }

    @Override
    public BillPersonnelAdmission queryEntity(String id) {
        BillPersonnelAdmission billPersonnelAdmission = this.getById(id);
        if(ObjectUtil.isEmpty(billPersonnelAdmission)) {
            throw new CommonException("入场申请不存在，id值为：{}", id);
        }
        return billPersonnelAdmission;
    }
}
