/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.energy.energyElectricity.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.energy.energyElectricity.entity.vo.*;
import vip.xiaonuo.modular.energy.energyElectricity.param.EnergyElectricityListParm;
import vip.xiaonuo.modular.energy.energyElectricity.param.EnergyElectricityUnitIdParam;
import vip.xiaonuo.modular.energy.energyElectricity.param.EnergyElectricityProportionParam;
import vip.xiaonuo.modular.energy.energyElectricity.param.EnergyElectricityTopParm;
import vip.xiaonuo.modular.energy.energyElectricity.service.EnergyElectricityService;

import javax.annotation.Resource;
import java.util.List;

/**
 * 能耗电表控制器
 *
 * @author newspaper
 * @date  2023/12/21 09:15
 */
@Api(tags = "能耗电表控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class EnergyElectricityController {

    @Resource
    private EnergyElectricityService energyElectricityService;

    /**
     * 获取用电占比
     *
     * @author newspaper
     * @date  2023/12/21 09:15
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取用电占比")
    @GetMapping("/energy/energyElectricity/getProportion")
    public CommonResult<EnergyElectricityProportionVo> getProportion(EnergyElectricityProportionParam energyElectricityProportionParam) {
        String type = energyElectricityProportionParam.getType();
        String orgId = energyElectricityProportionParam.getOrgId();
        return CommonResult.data(energyElectricityService.getProportion(orgId,type));
    }

    /**
     * 获取用电量趋势
     *
     * @author newspaper
     * @date  2023/12/21 14:54
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("获取用电量趋势")
    @GetMapping("/energy/energyElectricity/listTrend")
    public CommonResult<EnergyElectricityTrendVo> listTrend(EnergyElectricityListParm energyElectricityListParm) {
        return CommonResult.data(energyElectricityService.listTrend(energyElectricityListParm));
    }


    /**
     * 获取用电量排名
     *
     * @author newspaper
     * @date  2023/12/21 14:54
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("获取用电量排名")
    @GetMapping("/energy/energyElectricity/listTop")
    public CommonResult<List<EnergyElectricityTopVo>> listTop(EnergyElectricityTopParm energyElectricityTopParm) {
        String orgId = energyElectricityTopParm.getOrgId();
        return CommonResult.data(energyElectricityService.listTop(energyElectricityTopParm,orgId));
    }

    /**
     * 获取大屏能耗管理
     *
     * @author newspaper
     * @date  2023/12/21 14:54
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("获取大屏能耗管理")
    @GetMapping("/energy/energyElectricity/listScreenEnergy")
    public CommonResult<List<EnergyScreenVo>> listScreenEnergy(OrgIdParam orgIdParam) {
        String orgId = orgIdParam.getOrgId();
        return CommonResult.data(energyElectricityService.listScreenEnergy(orgId));
    }

    /**
     * 获取移动端能耗实时监测
     *
     * @author newspaper
     * @date  2023/12/21 14:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取移动端能耗实时监测")
    @PostMapping("/energy/energyElectricity/getToday")
    public CommonResult<EnergyElectricityMobileVo> getToday(@RequestBody EnergyElectricityUnitIdParam electricityUnitIdParam) {
        return CommonResult.data(energyElectricityService.getToday(electricityUnitIdParam));
    }

}
