/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.energy.energyThreshold.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.energy.energyThreshold.entity.vo.EnergyThresholdListVo;
import vip.xiaonuo.modular.energy.energyThreshold.param.*;
import vip.xiaonuo.modular.energy.energyThreshold.service.EnergyThresholdService;

import javax.annotation.Resource;
import java.util.List;

/**
 * 能耗阈值控制器
 *
 * @author newspaper
 * @date  2023/12/26 09:32
 */
@Api(tags = "能耗阈值控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class EnergyThresholdController {

    @Resource
    private EnergyThresholdService energyThresholdService;

    /**
     * 根据单元ID获取阈值
     *
     * @author newspaper
     * @date  2023/12/26 10:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取能耗阈值")
    @GetMapping("/energy/energyThreshold/getThreshold")
    public CommonResult<List<EnergyThresholdListVo>> getThreshold(EnergyThresholdOrgIdParam energyThresholdOrgIdParam) {
        return CommonResult.data(energyThresholdService.getThreshold(energyThresholdOrgIdParam));
    }

    /**
     * 编辑能耗阈值
     *
     * @author newspaper
     * @date  2023/12/26 10:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("编辑能耗阈值")
    @PostMapping("/energy/energyThreshold/saveThreshold")
    public CommonResult<String> saveThreshold(@RequestBody EnergyThresholdSaveParam energyThresholdSaveParam) {
        energyThresholdService.saveThreshold(energyThresholdSaveParam);
        return CommonResult.ok();
    }


    /**
     * 修改能耗阈值状态
     *
     * @author newspaper
     * @date  2023/12/26 10:20
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("修改能耗阈值状态")
    @PostMapping("/energy/energyThreshold/modifyThresholdStatus")
    public CommonResult<String> modifyThresholdStatus(@RequestBody EnergyThresholdModifyParam energyThresholdModifyParam) {
        energyThresholdService.modifyThresholdStatus(energyThresholdModifyParam);
        return CommonResult.ok();
    }

    /**
     * 同步能耗阈值
     *
     * @author newspaper
     * @date  2023/12/26 10:20
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("同步能耗阈值")
    @PostMapping("/energy/energyThreshold/syncThreshold")
    public CommonResult<String> syncThreshold(@RequestBody EnergyThresholdSyncParam energyThresholdSyncParam) {
        energyThresholdService.syncThreshold(energyThresholdSyncParam);
        return CommonResult.ok();
    }
}
