/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.feed.feedingthreshold.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.feed.feedingthreshold.param.AddFeedingParam;
import vip.xiaonuo.modular.feed.feedingthreshold.param.FeedingEditParam;
import vip.xiaonuo.modular.feed.feedingthreshold.param.FeedingThresholdAddParam;
import vip.xiaonuo.modular.feed.feedingthreshold.param.FeedingThresholdIdParam;
import vip.xiaonuo.modular.feed.feedingthreshold.service.FeedingThresholdService;
import vip.xiaonuo.modular.feed.feedingthreshold.vo.FeedingThresholdVo;
import vip.xiaonuo.modular.feed.feedingthreshold.vo.FeedingThresholdVo2;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 精准饲喂阈值控制器
 *
 * @author wang
 * @date 2024/01/02 10:37
 */
@Api(tags = "精准饲喂阈值控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class FeedingThresholdController {

    @Resource
    private FeedingThresholdService feedingThresholdService;

    /**
     * 编辑精准饲喂阈值
     *
     * @author wang
     * @date 2024/01/02 10:37
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("编辑精准饲喂猪只阈值")
    @CommonLog("编辑精准饲喂猪只阈值")
    @PostMapping("/feed/feedingthreshold/addAndEdit")
    public CommonResult<String> addAndEdit(@RequestBody @Valid FeedingThresholdAddParam feedingThresholdAddParam) {
        feedingThresholdService.addAndEdit(feedingThresholdAddParam);
        return CommonResult.ok();
    }

    @ApiOperationSupport(order = 2)
    @ApiOperation("展示精准饲喂猪只阈值")
    @CommonLog("展示精准饲喂猪只阈值")
    @PostMapping("/feed/feedingthreshold/listFat")
    public CommonResult<FeedingThresholdVo> listFat(@RequestBody @Valid FeedingThresholdIdParam feedingThresholdIdParam) {
        return CommonResult.data(feedingThresholdService.listFat(feedingThresholdIdParam.getOrgId()));
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation("同步精准饲喂料塔阈值")
    @CommonLog("同步精准饲喂料塔阈值")
    @PostMapping("/feed/feedingthreshold/addFeed")
    public CommonResult<String> addFeed(@RequestBody @Valid AddFeedingParam feedingParam) {
        feedingThresholdService.addFeed(feedingParam);
        return CommonResult.ok();
    }


    @ApiOperationSupport(order = 4)
    @ApiOperation("展示精准饲喂料塔阈值")
    @CommonLog("展示精准饲喂料塔阈值")
    @PostMapping("/feed/feedingthreshold/listFeeding")
    public CommonResult<List<FeedingThresholdVo2>> listFeeding(@RequestBody @Valid AddFeedingParam feedingParam) {
        return CommonResult.data(feedingThresholdService.listFeeding(feedingParam));
    }

    @ApiOperationSupport(order = 5)
    @ApiOperation("修改精准饲喂料塔阈值")
    @CommonLog("修改精准饲喂料塔阈值")
    @PostMapping("/feed/feedingthreshold/editFeeding")
    public CommonResult<String> editFeeding(@RequestBody @Valid FeedingEditParam feedingEditParam) {
        feedingThresholdService.editFeeding(feedingEditParam);
        return CommonResult.ok();
    }
}
